
function [newScenario, valid] = CloseRoads(closedRoads, oldScenario)

%
% create a new scenario
%

newScenario           = oldScenario; % start by cloning the old scenario
newScenario.INVISIBLE = closedRoads; % only for visualization
valid                 = true;        % for now, checked in the following

CLOSURE_CNT = sum(closedRoads);
LINK_CNT    = size(oldScenario.MU,2);
OPEN_CNT    = LINK_CNT - CLOSURE_CNT;

%
% tests for zero closed roads (=> valid) and zero open roads (=> not valid)
%

if (CLOSURE_CNT == 0)
	valid = true;
	return;
elseif (CLOSURE_CNT == LINK_CNT)
	valid = false;
	return;
end

%
% set all turning probabilities from and into closed links to zero
%

for i = 1 : LINK_CNT
    if closedRoads(i)
		newScenario.TURNS(i, :) = zeros(1, LINK_CNT + 1);
		newScenario.TURNS(:, i) = zeros(LINK_CNT, 1);
    end
end

%
% has every non-closed link at least one non-closed outgoing link?
%

for i = 1 : LINK_CNT
    if ~closedRoads(i)
		turnSum = sum(newScenario.TURNS(i, 1 : LINK_CNT));
        if (turnSum == 0)
			valid = false;
			return;
        end
    end
end

%
% can every non-closed link reach every other non-closed link?
% 

T = diag(ones(1, OPEN_CNT));
i_in_T = 1;
for i = 1 : LINK_CNT
    if ~closedRoads(i)
		j_in_T = 1;
        for j = 1 : LINK_CNT
            if ~closedRoads(j)
				T(i_in_T, j_in_T) = T(i_in_T, j_in_T) + newScenario.TURNS(i, j);                
                j_in_T=j_in_T+1;
            end
        end
		i_in_T=i_in_T+1;
    end
end
% now the following holds: T = 1 + P, only for non-closed links

R = diag(ones(1, OPEN_CNT)); 
for i = 1 : OPEN_CNT
	R = R * T;
end
% now the following holds: R(i,j) > 0 <=> i can reach j

for  i = 1 : OPEN_CNT
    for j = 1 : OPEN_CNT
        if R(i,j) == 0
			valid = false;
			return;
        end
    end
end
% now the following holds: the new scenario is valid

%
% re-normalize turning probabilities
%

for i = 1 : LINK_CNT
    if ~closedRoads(i)
		turnSum = sum(newScenario.TURNS(i, 1 : LINK_CNT));
		exits = newScenario.TURNS(i, LINK_CNT + 1);
		newScenario.TURNS(i, 1 : LINK_CNT) = newScenario.TURNS(i, 1 : LINK_CNT) .* (1.0 - exits) / turnSum; 
    end
end

%
% re-scale arrival rates to maintain total demand
%

movedLambda = 0;
for i = 1 : LINK_CNT
    if closedRoads(i)
		movedLambda = movedLambda + oldScenario.LAMBDA(i);
		newScenario.LAMBDA(i) = 0;
		newScenario.MU(i) = 0;
    end
end
movedLambda = movedLambda / (LINK_CNT - CLOSURE_CNT);

for i = 1 : LINK_CNT
    if ~closedRoads(i)
		newScenario.LAMBDA(i) = newScenario.LAMBDA(i) + movedLambda;
    end
end

end